/* UNH CS 758 assignment 7

   Parse a string using a context-free grammar.

   see parse.h for overview of major data structures
*/

#include <stdio.h>
#include <stdlib.h>  /* free, EXIT_SUCCESS, rand */
#include <string.h>  /* strtok */
#include <ctype.h>   /* islower */
#include "utils.h"
#include "parse.h"


/* declare functions used in main but defined elsewhere */

/* grammar.c */
cfg *read_grammar(FILE *infile);
void free_grammar(cfg *g);
void fprint_grammar(FILE *outfile, cfg *g);
token_seq *read_sentence(FILE *infile, cfg *g);
void fprint_sentence(FILE *outfile, token_seq *s, cfg *g);
void free_sentence(token_seq *s);
void fprint_parses(FILE *outfile, parses *ps, cfg *g);
void free_parses(parses *ps);
/* enum.c */
parses *enum_parse(token_seq *s, cfg *g);
/* cyk.c */
parses *cyk_parse(token_seq *s, cfg *g);


/************************************************************
 * arg parsing and main
 ************************************************************/


void usage(void)
{
    fprintf(stderr, "usage: parse <grammarfile> <algname>\n");
    fprintf(stderr,
	    "where <grammarfile> is the path of a file containing a CFG in CNF\n"
	    "  and <algname> is either dfs or cyk\n"
	    "The sentence to parse is read from stdin and a parse is written to stdout.\n");
    exit(EXIT_FAILURE);
}


int main(int argc, char *const argv[])
{
  FILE *infile;
  cfg *g;
  token_seq *s;
  parses *ps;

  if (argc != 3) usage();
  infile = fopen(argv[1], "r");
  if (!infile) panic ("can't open the specified grammar file");
  g = read_grammar(infile);
  /* fprint_grammar(stderr, g); */
  if (fclose(infile) != 0) panic("error closing grammar file");
  s = read_sentence(stdin, g);
  /* fprint_sentence(stderr, s, g); */
  if(strcmp(argv[2], "enum") == 0)
    ps = enum_parse(s, g);
  else if(strcmp(argv[2], "cyk") == 0)
    ps = cyk_parse(s, g);
  else
    usage();
  fprint_parses(stdout, ps, g);
  free_parses(ps);
  free_sentence(s);
  free_grammar(g);
  return EXIT_SUCCESS;
}


/* EOF */
