/* UNH CS 758 assignment 7

   Parse a string using a context-free grammar.

   re-written from scratch by Wheeler Ruml, October 2023

   uses symbol table when reading input to transform nonterminals and
   terminals to consistent unique integers.  this allows the parsing
   code to index rules in an array by symbol.  less importantly, it
   can compare ints rather than strings.

   possible improvements:
     more pruning in enumerative parser
     flag to print all or only one parse
     check grammar for reasonableness
     verbose flag to print debugging info?
*/


#include "symbol_table.h"


/* context-free grammar (CFG) in Chomsky normal form (CNF)

   two types of rules: non-terminal -> non-terminal-1 non-terminal-2
                       non-terminal -> terminal
   lrs = left-hand side, rhs = right-hand side
 */


#define START_NONTERMINAL_NAME "S"


/* node in linked list of all rhs nonterminal grammar rules whose lhs
   is a specific nonterminal */
typedef struct nonterminal_rhs {
  int nonterminal_1;
  int nonterminal_2;
  double probability;
  struct nonterminal_rhs *next;
} nonterminal_rhs;


/* node in linked list of all lhs nonterminals whose rhs is a specific
   terminal */
typedef struct terminal_lhs {
  int nonterminal;
  double probability;
  struct terminal_lhs *next;
} terminal_lhs;


typedef struct {
  symbol_table *st;  /* mapping string <-> symbol id */
  int size; /* size of these two arrays of linked lists */
  nonterminal_rhs **nonterminal_rules; /* indexed by lhs nonterminal */
  terminal_lhs **terminal_rules; /* indexed by rhs terminal */
} cfg;


/* sequence of terminal tokens to be parsed */
typedef struct {
  int size;
  int len;
  int *tokens; /* array of length size holding len tokens */
} token_seq;


/* a parse covers a sequence of terminals with a recursive
   decomposition of nonterminals.  all CNF CFG parses are binary
   branching until the nonterminals that cover the terminals. */
typedef struct parse {
  /* first == NULL, this is a terminal.
     second == NULL, this is a nonterminal and first will be a terminal.
     otherwise, a branching nonterminal. */
  int symbol;
  struct parse *first;
  struct parse *second;
  double probability; /* undefined iff this is a terminal */
} parse;


/* linked list of parses */
typedef struct parses {
  parse *parse;
  struct parses *next;
} parses;


/* defined in grammar.c, used in parsers */
void free_parse(parse *p);
parses *wrap_parse(parse *p);


/* EOF */
