/* C utilities

 */

#include <stdio.h>  /* fflish */
#include <stdlib.h>  /* perror, malloc */
#include <string.h>  /* strcpy, strlen */
#include "utils.h"


void panic(char *s)
{
  fflush(NULL);
  fprintf(stderr, "\n");
  perror(s);
  fflush(stderr);
  exit(EXIT_FAILURE);
}


/* returns a fresh heap-allocated copy of s */
char *strcpy_new(char *s)
{
  char *n = (char *) malloc(strlen(s)+1);
  return strcpy(n, s);
}


/* a combination of realloc and calloc: realloc p to hold newnobj
   objects, each of size bytes, and initializes the new space to zero
   bytes */
void *recalloc(void *p, size_t oldnobj, size_t newnobj, size_t size)
{
  size_t oldsize = oldnobj * size;
  size_t newsize = newnobj * size;
  char *newp = (char *) realloc(p, newsize);
  size_t i;
  
  if (!newp) panic("failed to realloc in recalloc");
  for (i = oldsize; i < newsize; i++) {
    newp[i] = 0;
  }
  return (void *) newp;
}


/* EOF */
